/***************************************************************************
                          glowclient.cpp  -  description
                             -------------------
    begin                : Thu Sep 6 2001
    copyright            : (C) 2001 by Henning Burchardt
    email                : h_burchardt@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qapplication.h>
#include <qbitmap.h>
#include <qimage.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <kconfig.h>
#include <kdebug.h>
#include <klocale.h>
#include <kpixmapeffect.h>
#include <kpixmap.h>
#include <kstddirs.h>
#include <kwin/options.h>
#include <kwin/workspace.h>
#include "resources.h"
#include "glowclient.h"
#include "glowbutton.h"

using namespace KWinInternal;


namespace Glow
{

static const int SIDE_MARGIN = 4;
static const int BOTTOM_MARGIN = 2;
static const int TITLE_MARGIN = 2;
static const int TITLE_SPACING = 1;
static const int RESIZE_HANDLE_WIDTH = 20;
static const int RESIZE_HANDLE_HEIGHT = 3;

//-----------------------------------------------------------------------------

GlowClientConfig::GlowClientConfig()
{
	themeName = "default";
}

void GlowClientConfig::load()
{
	KConfig conf("kwinglowrc");
	conf.setGroup("General");

	const QColor defaultCloseButtonColor(DEFAULT_CLOSE_BUTTON_COLOR);
	const QColor defaultMaximizeButtonColor(DEFAULT_MAXIMIZE_BUTTON_COLOR);
	const QColor defaultIconifyButtonColor(DEFAULT_ICONIFY_BUTTON_COLOR);
	const QColor defaultHelpButtonColor(DEFAULT_HELP_BUTTON_COLOR);
	const QColor defaultStickyButtonColor(DEFAULT_STICKY_BUTTON_COLOR);

	stickyButtonGlowColor = conf.readColorEntry(
		"stickyButtonGlowColor", &defaultStickyButtonColor);

	helpButtonGlowColor = conf.readColorEntry(
		"helpButtonGlowColor", &defaultHelpButtonColor);

	iconifyButtonGlowColor = conf.readColorEntry(
		"iconifyButtonGlowColor", &defaultIconifyButtonColor);

	maximizeButtonGlowColor = conf.readColorEntry(
		"maximizeButtonGlowColor", &defaultMaximizeButtonColor);

	closeButtonGlowColor = conf.readColorEntry(
		"closeButtonGlowColor", &defaultCloseButtonColor);

	showResizeHandle = conf.readBoolEntry("showResizeHandle", true);
	titlebarGradientType = conf.readNumEntry("titlebarGradientType",
		KPixmapEffect::DiagonalGradient);

	themeName = conf.readEntry ("themeName", "default");
}

//-----------------------------------------------------------------------------

GlowClientGlobals *GlowClientGlobals::m_instance = 0;

GlowClientGlobals *GlowClientGlobals::instance()
{
	if( ! m_instance )
	{
		m_instance = new GlowClientGlobals();
	}
	return m_instance;
}

GlowClientGlobals::~GlowClientGlobals()
{
	deletePixmaps();
	m_instance = 0;
}

QString GlowClientGlobals::getPixmapName(PixmapType type, bool isActive)
{
	QString s = getPixmapTypeName(static_cast<PixmapType>(type));
	s += "|";
	s += isActive ? "Active" : "NotActive";
	return s;
}

GlowClientGlobals::GlowClientGlobals()
	: QObject()
{
	buttonFactory = new GlowButtonFactory();
	readConfig();
	readTheme ();
	if (!createPixmaps())
	{
		deletePixmaps();
		delete theme;
		config->themeName = "default";
		readTheme();
		createPixmaps();
	}
}

void GlowClientGlobals::readConfig()
{
	this->config = new GlowClientConfig();
	config->load();
}

void GlowClientGlobals::readTheme()
{
	QString theme_config_file = KGlobal::dirs()->findResource ("data",
			QString("kwin/glow-themes/") + config->themeName + "/" +
			config->themeName + ".theme");
	if (theme_config_file == QString::null)
	{
		config->themeName = "default";
		return;
	}

	KConfig conf (theme_config_file);
	this->theme = new GlowTheme(default_glow_theme);
	
	theme->buttonSize = conf.readSizeEntry ("buttonSize",
			&theme->buttonSize);
	theme->stickyOnPixmap = conf.readEntry ("stickyOnPixmap",
			theme->stickyOnPixmap);
	theme->stickyOffPixmap = conf.readEntry ("stickyOffPixmap",
			theme->stickyOffPixmap);
	theme->maximizeOnPixmap = conf.readEntry ("maximizeOnPixmap",
			theme->maximizeOnPixmap);
	theme->maximizeOffPixmap = conf.readEntry ("maximizeOffPixmap",
			theme->maximizeOffPixmap);
	theme->helpPixmap = conf.readEntry ("helpPixmap",
			theme->helpPixmap);
	theme->closePixmap = conf.readEntry ("closePixmap",
			theme->closePixmap);
	theme->iconifyPixmap = conf.readEntry ("iconifyPixmap",
			theme->iconifyPixmap);
	theme->stickyOnGlowPixmap = conf.readEntry ("stickyOnGlowPixmap",
			theme->stickyOnGlowPixmap);
	theme->stickyOffGlowPixmap = conf.readEntry ("stickyOffGlowPixmap",
			theme->stickyOffGlowPixmap);
	theme->maximizeOnGlowPixmap = conf.readEntry ("maximizeOnGlowPixmap",
			theme->maximizeOnGlowPixmap);
	theme->maximizeOffGlowPixmap = conf.readEntry ("maximizeOffGlowPixmap",
			theme->maximizeOffGlowPixmap);
	theme->helpGlowPixmap = conf.readEntry ("helpGlowPixmap",
			theme->helpGlowPixmap);
	theme->closeGlowPixmap = conf.readEntry ("closeGlowPixmap",
			theme->closeGlowPixmap);
	theme->iconifyGlowPixmap = conf.readEntry ("iconifyGlowPixmap",
			theme->iconifyGlowPixmap);
}

void GlowClientGlobals::reset()
{
	deletePixmaps();
	delete config;
	readConfig();
	delete theme;
	readTheme ();
	if (! createPixmaps())
	{
		deletePixmaps();
		delete theme;
		config->themeName = "default";
		readTheme();
		createPixmaps();
	}
}

bool GlowClientGlobals::createPixmaps()
{	
	for( int type=0; type<=Close; type++ ) {
		if (! createPixmap(static_cast<PixmapType>(type), false))
			return false;
		if (! createPixmap(static_cast<PixmapType>(type), true))
			return false;
	}

	return true;
}

void GlowClientGlobals::deletePixmaps()
{
	PixmapCache::clear();
}

const QString GlowClientGlobals::getPixmapTypeName(PixmapType type)
{
	switch(type) {
		case (StickyOn):
			return "StickyOn";
		case(StickyOff ):
			return "StickyOff";
		case(Help):
			return "Help";
		case(Iconify):
			return "Iconify";
		case(MaximizeOn):
			return "MaximizeOn";
		case(MaximizeOff):
			return "MaximizeOff";
		case(Close):
			return "Close";
		default:
			return QString::null;
	}
}

bool GlowClientGlobals::createPixmap(PixmapType type, bool isActive)
{
	QString theme_dir = KGlobal::dirs()->findResource ("data",
			QString("kwin/glow-themes/") + config->themeName + "/");

	QColor glow_color;
	QColor color = options->color(Options::ButtonBg, isActive);

	QImage bg_image (theme_dir+theme->backgroundPixmap);
	QImage bg_alpha_image (theme_dir+theme->backgroundAlphaPixmap);
	QImage fg_image;
	QImage glow_image;
	
	switch(type) { 
		case (StickyOn):
		{
			fg_image = QImage (theme_dir+theme->stickyOnPixmap);
			glow_image = QImage (theme_dir+theme->stickyOnGlowPixmap);
			glow_color = config->stickyButtonGlowColor;
			break;
		}
		case (StickyOff):
		{
			fg_image = QImage (theme_dir+theme->stickyOffPixmap);
			glow_image = QImage (theme_dir+theme->stickyOffGlowPixmap);
			glow_color = config->stickyButtonGlowColor;
			break;
		}
		case (Help):
		{
			fg_image = QImage (theme_dir+theme->helpPixmap);
			glow_image = QImage (theme_dir+theme->helpGlowPixmap);
			glow_color = config->helpButtonGlowColor;
			break;
		}
		case (Iconify):
		{
			fg_image = QImage (theme_dir+theme->iconifyPixmap);
			glow_image = QImage (theme_dir+theme->iconifyGlowPixmap);
			glow_color = config->iconifyButtonGlowColor;
			break;
		}
		case (MaximizeOn):
		{
			fg_image = QImage (theme_dir+theme->maximizeOnPixmap);
			glow_image = QImage (theme_dir+theme->maximizeOnGlowPixmap);
			glow_color = config->maximizeButtonGlowColor;
			break;
		}
		case (MaximizeOff):
		{
			fg_image = QImage (theme_dir+theme->maximizeOffPixmap);
			glow_image = QImage (theme_dir+theme->maximizeOffGlowPixmap);
			glow_color = config->maximizeButtonGlowColor;
			break;
		}
		case (Close):
		{
			fg_image = QImage (theme_dir+theme->closePixmap);
			glow_image = QImage (theme_dir+theme->closeGlowPixmap);
			glow_color = config->closeButtonGlowColor;
			break;
		}
	}

	if (bg_image.size() != theme->buttonSize
			|| bg_alpha_image.size() != theme->buttonSize
			|| fg_image.size() != theme->buttonSize
			|| glow_image.size() != theme->buttonSize)
		return false;

	QPixmap * glowPm = buttonFactory->createGlowButtonPixmap(
		bg_image, bg_alpha_image, fg_image, glow_image,
		color, glow_color);
	if (glowPm->isNull())
		return false;
	PixmapCache::insert(getPixmapName(type, isActive), glowPm);
	return true;
}

//-----------------------------------------------------------------------------

GlowClient::GlowClient(KWinInternal::Workspace *ws, WId w, QWidget *parent, const char* name )
	: KWinInternal::Client(ws, w, parent, name),
		m_stickyButton(0), m_helpButton(0), m_minimizeButton(0),
		m_maximizeButton(0), m_closeButton(0),
		m_mainLayout(0), m_leftButtonLayout(0), m_rightButtonLayout(0)
{
	_title_widget = new QWidget (this, "title_widget");
		
	createButtons();
	resetLayout();
	repaint();
}

GlowClient::~GlowClient()
{
}

void GlowClient::resizeEvent( QResizeEvent *e )
{
	Client::resizeEvent(e);
	doShape();
	repaint(false);
}

void GlowClient::paintEvent( QPaintEvent *e )
{
	Client::paintEvent(e);

	GlowClientConfig *conf = GlowClientGlobals::instance()->config;
	QRect r_this = rect();
	QRect r_title = m_titleSpacer->geometry();
	QColorGroup titleCg = options->colorGroup(Options::TitleBar, isActive());
	QColorGroup titleBlendCg = options->colorGroup(Options::TitleBlend, isActive());
	QColorGroup cg = colorGroup();
	QColor titleColor = options->color(Options::TitleBar, isActive());
	QColor titleBlendColor = options->color(Options::TitleBlend, isActive());
	QPainter p;
	QPointArray pArray, pArray2, pArray3;

	// pixmap for title bar
	QSize tBSize(width(), r_title.height());
	QSize gradientPixmapSize (tBSize-QSize(3,3));
	if (! gradientPixmapSize.isValid())
		gradientPixmapSize = QSize(0,0);
	KPixmap gradientPixmap(gradientPixmapSize);
	if (! gradientPixmapSize.isNull())
		KPixmapEffect::gradient(gradientPixmap, titleColor, titleBlendColor,
			(KPixmapEffect::GradientType) conf->titlebarGradientType);
	
	QPixmap titleBuffer(tBSize);
	p.begin(&titleBuffer);
	if (! gradientPixmap.isNull())
		p.drawPixmap(2, 2, gradientPixmap);
	
	if (tBSize.width()>=3 && tBSize.height()>=3)
	{
		// draw caption
		p.setFont(options->font(isActive()));
		p.setPen(options->color(Options::Font, isActive()));
		p.drawText(r_title.x(), 0,
			r_title.width(), r_title.height(),
			Qt::AlignLeft | Qt::AlignVCenter | Qt::SingleLine, caption());
		
		// draw borders
		pArray = QPointArray(3);
		pArray.setPoint(0, tBSize.width()-1, tBSize.height()/2-1);
		pArray.setPoint(1, r_title.x()+r_title.width()-1+tBSize.height()/2,
			tBSize.height()/2-1);
		pArray.setPoint(2, r_title.x()+r_title.width()-1, tBSize.height()-1);
		p.setPen(titleCg.mid());
		p.drawPolyline(pArray);
		p.drawLine (0, tBSize.height()-1, tBSize.width()-1, tBSize.height()-1);

		pArray2 = QPointArray(3);
		pArray2.setPoint(0, 1, tBSize.height()-2);
		pArray2.setPoint(1, 1, 1);
		pArray2.setPoint(2, tBSize.width()-2, 1);
		p.setPen(titleCg.light());
		p.drawPolyline(pArray2);
	}	

	pArray3 = QPointArray(4);
	pArray3.setPoint(0, 0, tBSize.height()-1);
	pArray3.setPoint(1, 0, 0);
	pArray3.setPoint(2, tBSize.width()-1, 0);
	pArray3.setPoint(3, tBSize.width()-1, tBSize.height()-1);
	p.setPen(Qt::black);
	p.drawPolyline(pArray3);
	p.end();

	_title_widget->setPaletteBackgroundPixmap (titleBuffer);
	
	p.begin(this);
	p.setPen(Qt::black);
	p.drawLine(0,tBSize.height(),0,r_this.height()-1);
	p.drawLine(0,r_this.height()-1,r_this.width()-1,r_this.height()-1);
	p.drawLine(r_this.width()-1,r_this.height()-1,
		r_this.width()-1,tBSize.height()/2);
	p.end();

	// paint resize handle if necessary
	if(conf->showResizeHandle && !isTool() && isResizable()
			&& width()>=2 && height() >= RESIZE_HANDLE_HEIGHT)
	{
		p.begin(this);
		p.setPen (Qt::black);
		p.drawLine (1, height() - RESIZE_HANDLE_HEIGHT,
			width()-2, height() - RESIZE_HANDLE_HEIGHT);
		p.end();
	}
}

void GlowClient::showEvent( QShowEvent *e )
{
	Client::showEvent(e);
	doShape();
	repaint(false);
}

void GlowClient::mouseDoubleClickEvent( QMouseEvent *e )
{
	if( m_titleSpacer->geometry().contains(e->pos()) )
		workspace()->performWindowOperation(
			this, options->operationTitlebarDblClick());
}

void GlowClient::activeChange(bool)
{
	updateButtonPixmaps();
	repaint(false);
}

void GlowClient::iconChange()
{
	// we have no (t yet an) icon button, so do nothing
}

void GlowClient::stickyChange(bool on)
{
	if (on) {
		m_stickyButton->setPixmapName(
			GlowClientGlobals::instance()->getPixmapName(
				GlowClientGlobals::StickyOn, isActive()));
		m_stickyButton->setTipText(i18n("Un-Sticky"));
	} else {
		m_stickyButton->setPixmapName(
			GlowClientGlobals::instance()->getPixmapName(
				GlowClientGlobals::StickyOff, isActive()));
		m_stickyButton->setTipText(i18n("Sticky"));
	}
}

void GlowClient::maximizeChange(bool on)
{
    if (on) {
		m_maximizeButton->setPixmapName(
			GlowClientGlobals::instance()->getPixmapName(
				GlowClientGlobals::MaximizeOn, isActive()));
		m_maximizeButton->setTipText(i18n("Restore"));
	} else {
		m_maximizeButton->setPixmapName(
			GlowClientGlobals::instance()->getPixmapName(
				GlowClientGlobals::MaximizeOff, isActive()));
		m_maximizeButton->setTipText(i18n("Maximize"));
	}
}

Client::MousePosition GlowClient::mousePosition(const QPoint &pos) const
{
	Client::MousePosition m = Nowhere;
	if( GlowClientGlobals::instance()->config->showResizeHandle && !isTool() ) {
		if( pos.y() >= height()-RESIZE_HANDLE_HEIGHT) {
			if( pos.x() < RESIZE_HANDLE_WIDTH) {
				m = BottomLeft;
			} else if( pos.x() > width() - RESIZE_HANDLE_WIDTH) {
				m = BottomRight;
			} else {
				m = Bottom;
			}
		} else {
			m = Client::mousePosition(pos);
		}
	} else {
		m = Client::mousePosition(pos);
	}
	return m;
}

void GlowClient::createButtons()
{
	GlowClientGlobals *globals = GlowClientGlobals::instance();
	GlowButtonFactory *factory = globals->buttonFactory;
	QSize size = globals->theme->buttonSize;

	m_stickyButton = factory->createGlowButton(_title_widget,
					"StickyButton", i18n("Sticky"));
	m_stickyButton->setFixedSize(size);
	connect(m_stickyButton, SIGNAL(clicked()), this, SLOT(toggleSticky()));
	m_buttonList.insert(m_buttonList.end(), m_stickyButton);

	m_helpButton = factory->createGlowButton(_title_widget,
					"HelpButton", i18n("Help"));
	m_helpButton->setFixedSize(size);
	connect(m_helpButton, SIGNAL(clicked()), this, SLOT(contextHelp()));
	m_buttonList.insert(m_buttonList.end(), m_helpButton);

	m_minimizeButton = factory->createGlowButton(_title_widget,
					"IconifyButton", i18n("Minimize"));
	m_minimizeButton->setFixedSize(size);
	connect(m_minimizeButton, SIGNAL(clicked()), this, SLOT(iconify()));
	m_buttonList.insert(m_buttonList.end(), m_minimizeButton);

	m_maximizeButton=factory->createGlowButton(_title_widget,
					"MaximizeButton", i18n("Maximize"));
	m_maximizeButton->setFixedSize(size);
	connect(m_maximizeButton, SIGNAL(clicked(int)), this, SLOT(slotMaximize(int)));
	m_buttonList.insert(m_buttonList.end(), m_maximizeButton);

	m_closeButton = factory->createGlowButton(_title_widget,
					"CloseButton", i18n("Close"));
	m_closeButton->setFixedSize(size);
	connect(m_closeButton, SIGNAL(clicked()), this, SLOT(closeWindow()));
	m_buttonList.insert(m_buttonList.end(), m_closeButton);
}

void GlowClient::resetLayout()
{
	if( m_mainLayout )
		delete m_mainLayout;
	m_mainLayout = new QVBoxLayout(this, 0, 0);
	m_mainLayout->addWidget (_title_widget);

	// update button positions and colors
	updateButtonPositions();
	updateButtonPixmaps();

	QHBoxLayout *topLayout = new QHBoxLayout(_title_widget, 0, 0, 0);
	topLayout->setMargin(0);
	topLayout->setSpacing(TITLE_SPACING);
	topLayout->addSpacing(SIDE_MARGIN);
	QVBoxLayout *outerLeftLayout = new QVBoxLayout(topLayout);
	outerLeftLayout->addSpacing(TITLE_MARGIN);
	outerLeftLayout->addItem(m_leftButtonLayout);
	outerLeftLayout->addSpacing(1);
	topLayout->addSpacing(SIDE_MARGIN);

	m_titleSpacer = new QSpacerItem(0, 0,
		QSizePolicy::Expanding, QSizePolicy::Expanding);
	topLayout->addItem(m_titleSpacer);

	topLayout->addSpacing(SIDE_MARGIN);
	QVBoxLayout *outerRightLayout = new QVBoxLayout(topLayout);
	outerRightLayout->addSpacing(TITLE_MARGIN);
	outerRightLayout->addItem(m_rightButtonLayout);
	outerRightLayout->addSpacing(1);
	topLayout->addSpacing(SIDE_MARGIN);

	QBoxLayout *midLayout = new QBoxLayout(m_mainLayout, QBoxLayout::LeftToRight, 0, 0);
	midLayout->addSpacing(SIDE_MARGIN);
	midLayout->addWidget(windowWrapper());
	midLayout->addSpacing(SIDE_MARGIN);

	if( GlowClientGlobals::instance()->config->showResizeHandle
		&& !isTool() && isResizable() ) {
		m_mainLayout->addSpacing(RESIZE_HANDLE_HEIGHT);
	} else {
		m_mainLayout->addSpacing(BOTTOM_MARGIN);
	}
	m_mainLayout->setStretchFactor(topLayout, 0);
	m_mainLayout->setStretchFactor(midLayout, 1);
}

void GlowClient::updateButtonPositions()
{
	QString buttons = options->titleButtonsLeft() + "|"
		+ options->titleButtonsRight();
	bool leftButtons=true;

	// hide all buttons
	for( unsigned int i=0; i<m_buttonList.size(); i++ )
		m_buttonList[i]->hide();

	m_leftButtonList.clear();
	m_rightButtonList.clear();

	// reset left and right button layout
	if(m_leftButtonLayout)
		delete m_leftButtonLayout;
	m_leftButtonLayout = new QBoxLayout(0, QBoxLayout::LeftToRight, 0, 0, 0);
	m_leftButtonLayout->setMargin(0);
	m_leftButtonLayout->setSpacing(TITLE_SPACING);
	if(m_rightButtonLayout)
		delete m_rightButtonLayout;
	m_rightButtonLayout = new QBoxLayout(0, QBoxLayout::LeftToRight, 0, 0, 0);
	m_rightButtonLayout->setMargin(0);
	m_rightButtonLayout->setSpacing(TITLE_SPACING);

	for( unsigned int i=0; i<buttons.length(); i++ )
	{
		char c = buttons[i].latin1();
		GlowButton *button = 0;
		if( c=='S' ) // sticky
			button = m_stickyButton;
		else if( c=='H' && providesContextHelp() ) // help
			button = m_helpButton;
		else if( c=='I' && isMinimizable() ) // iconify
			button = m_minimizeButton;
		else if( c=='A' && isMaximizable() ) // maximize
			button = m_maximizeButton;
		else if( c=='X' && isCloseable() ) // close
			button= m_closeButton;
		else if( c=='_' ) // spacer item
		{
			if(leftButtons)
				m_leftButtonLayout->addSpacing(4);
			else
				m_rightButtonLayout->addSpacing(4);
		}
		else if( c=='|' )
			leftButtons = false;

		if (button) {
			button->show(); // show visible buttons
			if (leftButtons) {
				m_leftButtonList.insert(m_leftButtonList.end(), button);
				m_leftButtonLayout->addWidget(button);
			} else {
				m_rightButtonList.insert(m_rightButtonList.end(), button);
				m_rightButtonLayout->addWidget(button);
			}
		}
	}
}

void GlowClient::updateButtonPixmaps()
{
	GlowClientGlobals *globals = GlowClientGlobals::instance();

	if ( isSticky() ) {
		m_stickyButton->setPixmapName(globals->getPixmapName(
			GlowClientGlobals::StickyOn, isActive()));
	} else {
		m_stickyButton->setPixmapName(globals->getPixmapName(
			GlowClientGlobals::StickyOff, isActive()));
	}
	m_helpButton->setPixmapName(globals->getPixmapName(
		GlowClientGlobals::Help, isActive()));

	m_minimizeButton->setPixmapName(globals->getPixmapName(
		GlowClientGlobals::Iconify, isActive()));

	if ( isMaximized() ) {
		m_maximizeButton->setPixmapName(globals->getPixmapName(
			GlowClientGlobals::MaximizeOn, isActive()));
	} else {
		m_maximizeButton->setPixmapName(globals->getPixmapName(
			GlowClientGlobals::MaximizeOff, isActive()));
	}
	m_closeButton->setPixmapName(globals->getPixmapName(
		GlowClientGlobals::Close, isActive()));
}

void GlowClient::doShape()
{
	QRegion mask(rect());
	// edges

	mask -= QRegion(width()-1,0,1,1);
	mask -= QRegion(0,height()-1,1,1);
	mask -= QRegion(width()-1,height()-1,1,1);
	setMask(mask);
}

bool GlowClient::isLeft(GlowButton *button)
{
	for( unsigned int i=0; i<m_leftButtonList.size(); i++ )
		if( m_leftButtonList[i] == button )
			return true;
	return false;
}

bool GlowClient::isRight(GlowButton *button)
{
	for( unsigned int i=0; i<m_rightButtonList.size(); i++ )
		if( m_rightButtonList[i] == button )
			return true;
	return false;
}

void GlowClient::slotMaximize(int button)
{
	if(button == QMouseEvent::RightButton)
		maximize(MaximizeHorizontal);
	else if(button == QMouseEvent::MidButton)
		maximize(MaximizeVertical);
	else // if(button == QMouseEvent::LeftButton)
		maximize(MaximizeFull);
}

}

extern "C"
{
	Client * allocate(Workspace * ws, WId w)
	{
		return new Glow::GlowClient(ws, w);
	}

	void init()
	{
		Glow::GlowClientGlobals::instance();
	}

	void reset()
	{
		Glow::GlowClientGlobals::instance()->reset();
		Workspace::self()->slotResetAllClientsDelayed();
	}

	void deinit()
	{
		delete Glow::GlowClientGlobals::instance();
	}
}

#include "glowclient.moc"

